-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------

NUM_SPELL_SLOTS = 10;
VERSION = "2.3.0.9"

-------------------------------------------------------------------------------
-- Variables
-------------------------------------------------------------------------------

SHOW_WELCOME = true;
FLOASPECTBAR_LAYOUT = "auto";
FLOASPECTBAR_SCALE = 1;
FLOASPECTBAR_BUTTONS_ORDER = {};

-------------------------------------------------------------------------------
-- Functions
-------------------------------------------------------------------------------

-- Executed on load, calls general set-up functions
function FloAspectBar_OnLoad()

	-- Class-based setup, abort if not supported
	local temp, classFileName = UnitClass("player");
	classFileName = strupper(classFileName);

	classSpells = FLO_ASPECT_SPELLS[classFileName];
	if ( classSpells == nil ) then
		return;
	end

	-- Store the spell list for later
	this.AvailableSpells = classSpells;
	if ( this.AvailableSpells == nil ) then
		return;
	end

	this.Spells = {};

	-- Cosmetic
	this:SetBackdropBorderColor(0.5, 0.5, 0.5);
	this:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b, 0.7);

	if ( SHOW_WELCOME ) then
		DEFAULT_CHAT_FRAME:AddMessage( "FloAspectBar "..VERSION.." loaded." );
		SHOW_WELCOME = nil;

		SLASH_FLOASPECTBAR1 = "/floaspectbar";
		SLASH_FLOASPECTBAR2 = "/fab";
		SlashCmdList["FLOASPECTBAR"] = FloAspectBar_ReadCmd;

		this:RegisterEvent("VARIABLES_LOADED");
	end

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("LEARNED_SPELL_IN_TAB");
	this:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	this:RegisterEvent("SPELL_UPDATE_USABLE");
	this:RegisterEvent("PLAYER_AURAS_CHANGED");
	this:RegisterEvent("UPDATE_BINDINGS");
end

function FloAspectBar_OnEvent()

	if ( event == "PLAYER_ENTERING_WORLD" or event == "LEARNED_SPELL_IN_TAB" ) then
		FloAspectBar_Setup();
	elseif ( event == "VARIABLES_LOADED" ) then

		FloAspectBar_SetLayout(FLOASPECTBAR_LAYOUT);
		FloAspectBar_SetScale(FLOASPECTBAR_SCALE);

		-- Hook the UIParent_ManageFramePositions function
		hooksecurefunc("UIParent_ManageFramePositions", FloAspectBar_UpdatePosition);
	elseif ( event == "UPDATE_BINDINGS" ) then
		FloLib_UpdateBindings(this, "SHAPESHIFT");
	else
		FloAspectBar_UpdateState();
	end
end

function FloAspectBar_ReadCmd(line)

	local cmd, var = strsplit(' ', line or "");

	if ( cmd == "scale" and tonumber(var) ) then
		FloAspectBar_SetScale(var);
	elseif ( cmd == "lock" or cmd == "unlock" or cmd == "auto" ) then
		FloAspectBar_SetLayout(cmd);
	else
		DEFAULT_CHAT_FRAME:AddMessage( "FloAspectBar usage :" );
		DEFAULT_CHAT_FRAME:AddMessage( "/fab lock|unlock : lock/unlock position" );
		DEFAULT_CHAT_FRAME:AddMessage( "/fab auto : Automatic positioning" );
		DEFAULT_CHAT_FRAME:AddMessage( "/fab scale <num> : Set scale" );
		return;
	end
end

function FloAspectBar_ReceiveDrag(releaseCursor)

	if ( InCombatLockdown() ) then
		return;
	end

	local cursorType, index, info;

	cursorType, index, info = GetCursorInfo();

	if ( cursorType ~= "spell" or info ~= BOOKTYPE_SPELL ) then
		return;
	end

	local button = this;
	local newspell = GetSpellName(index, info);
	this = this:GetParent();

	-- find the spell in the curent list
	for i = 1, #this.AvailableSpells do
		if ( this.AvailableSpells[i].name == newspell ) then
			if ( releaseCursor ) then
				ClearCursor();
			end
			FloLib_Swap(FLOASPECTBAR_BUTTONS_ORDER, FLOASPECTBAR_BUTTONS_ORDER[button:GetID()], i);
			FloAspectBar_Setup();
			break;
		end
	end

end

function FloAspectBar_SetupSpell(spell, spellID, pos)

	local durationMod = 0;
	local lifeMod = 0;
	local algo, lifeTable;
	local button, icon, texture;

	button = getglobal(this:GetName().."Button"..pos);
	icon = getglobal(this:GetName().."Button"..pos.."Icon");

	button:SetAttribute("type1", "spell");
	button:SetAttribute("spell", spell.name);

	texture = GetSpellTexture(spellID, BOOKTYPE_SPELL);
	icon:SetTexture(texture);

	this.Spells[pos] = { name = spell.name, id = spellID };

end

function FloAspectBar_Setup()

	local numSpells = 0;
	local button, spellID, spell;
	local i = 1;

	this.Spells = {};

	-- Check already positionned spells
	while ( FLOASPECTBAR_BUTTONS_ORDER[i] ) do

		local n = FLOASPECTBAR_BUTTONS_ORDER[i];

		spell = this.AvailableSpells[n];
		spellID = FloAspectBar_GetSpellID(spell.name);
		if ( spellID > -1 ) then
			FloAspectBar_SetupSpell(spell, spellID, i);
			i = i + 1;
		else
			-- this spell is unavailable, shift the remaining indexes by 1
			for j = i, #FLOASPECTBAR_BUTTONS_ORDER do
				FLOASPECTBAR_BUTTONS_ORDER[j] = FLOASPECTBAR_BUTTONS_ORDER[j+1];
			end
		end

	end

	numSpells = i - 1;

	for n = 1, #this.AvailableSpells do

		if ( n > NUM_SPELL_SLOTS ) then
			break;
		end

		-- Check if this spell is already positionned
		i = nil;
		for j = 1, #FLOASPECTBAR_BUTTONS_ORDER do
			if ( FLOASPECTBAR_BUTTONS_ORDER[j] == n ) then
				i = 1;
				break;
			end
		end

		if ( not i ) then
			spell = this.AvailableSpells[n];
			spellID = FloAspectBar_GetSpellID(spell.name);
			if ( spellID > -1 ) then

				numSpells = numSpells + 1;

				FloAspectBar_SetupSpell(spell, spellID, numSpells);
				FLOASPECTBAR_BUTTONS_ORDER[numSpells] = n;
			end
		end
	end

	if ( numSpells > 0 ) then

		this:Show();
		this:SetWidth(numSpells * 35 + 12);

		for i=1, NUM_SPELL_SLOTS do

			button = getglobal(this:GetName().."Button"..i);

			if ( i <= numSpells ) then
				button:Show();
			else
				button:Hide();
			end
		end
	else
		this:Hide();
	end
	FloAspectBar_UpdateState();
end

function FloAspectBar_UpdateState()

	local numSpells = #this.Spells;
	local button, spell, cooldown, normalTexture, icon;
	local start, duration, enable;

	for i=1, numSpells do

		button = getglobal(this:GetName().."Button"..i);

		spell = this.Spells[i];

		if ( GetPlayerBuffName(spell.name) ) then
			button:SetChecked(1);
		else
			button:SetChecked(0);
		end

		--Cooldown stuffs
		cooldown = getglobal(this:GetName().."Button"..i.."Cooldown");
		start, duration, enable = GetSpellCooldown(spell.id, BOOKTYPE_SPELL);
		CooldownFrame_SetTimer(cooldown, start, duration, enable);

		--Castable stuffs
		normalTexture = getglobal(this:GetName().."Button"..i.."NormalTexture");
		icon = getglobal(this:GetName().."Button"..i.."Icon");
		isUsable, noMana = IsUsableSpell(spell.id, BOOKTYPE_SPELL);

		if ( isUsable ) then
			icon:SetVertexColor(1.0, 1.0, 1.0);
			normalTexture:SetVertexColor(1.0, 1.0, 1.0);
		elseif ( noMana ) then
			icon:SetVertexColor(0.5, 0.5, 1.0);
			normalTexture:SetVertexColor(0.5, 0.5, 1.0);
		else
			icon:SetVertexColor(0.4, 0.4, 0.4);
			normalTexture:SetVertexColor(1.0, 1.0, 1.0);
		end
	end
end

function FloAspectBar_UpdatePosition()

	if ( FLOASPECTBAR_LAYOUT ~= "auto" ) then
		return;
	end

	local yOffset = -3;
	local anchorFrame;

	if ( not MainMenuBar:IsShown() ) then
		anchorFrame = "UIParent";
		yOffset = 110-UIParent:GetHeight();
	else
		anchorFrame = "MainMenuBar";
		if ( ReputationWatchBar:IsShown() and MainMenuExpBar:IsShown() ) then
			yOffset = yOffset + 9;
		end

		if ( MainMenuBarMaxLevelBar:IsShown() ) then
			yOffset = yOffset - 5;
		end

		if ( SHOW_MULTI_ACTIONBAR_2 ) then
			yOffset = yOffset + 45;
		end
	end

	FloAspectBar:ClearAllPoints();
	FloAspectBar:SetPoint("BOTTOMLEFT", anchorFrame, "TOPLEFT", 512/FLOASPECTBAR_SCALE, yOffset/FLOASPECTBAR_SCALE);
end

function FloAspectBar_SetLayout(mode)

	local unlocked = (mode == "unlock");

	FLOASPECTBAR_LAYOUT = mode;
	DEFAULT_CHAT_FRAME:AddMessage( "FloAspectBar layout "..FLOASPECTBAR_LAYOUT );

	FloAspectBar:EnableMouse(unlocked);
	if ( unlocked ) then
		FloAspectBar:RegisterForDrag("LeftButton");
	end

	if ( mode == "auto" ) then
		-- Force the auto positionning
		FloAspectBar_UpdatePosition();
	end
end

function FloAspectBar_SetScale(scale)

	scale = tonumber(scale);
	if ( not scale or scale <= 0 ) then
		DEFAULT_CHAT_FRAME:AddMessage( "FloAspectBar : scale must be >0 ("..scale..")" );
		return;
	end

	FLOASPECTBAR_SCALE = scale;

	local v = FloAspectBar;
	local p, a, rp, ox, oy = v:GetPoint();
	local os = v:GetScale();
	v:SetScale(scale);
	if ( a == nil or a == UIParent or a == MainMenuBar ) then
		v:SetPoint(p, a, rp, ox*os/scale, oy*os/scale);
	end

end

-- Returns the ID of the given aspect spell from the spellbook
function FloAspectBar_GetSpellID(aspect)

	local i = 1;
	local valid = -1;
	local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
	while spellName do
		if ( spellName == aspect) then
			valid = i;
		elseif ( valid > -1 ) then
			return valid;
		end
		i = i + 1;
		spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
	end
	return valid;
end

-- Returns the name, the texture and active state of an aspect
function FloAspectBar_GetAspectInfo(id)
	for spell,i in pairs(FloAspectBar_FormID) do
		if ( i == id ) then
			local texture = GetSpellTexture(FloAspectBar_GetAspectID(spell), BOOKTYPE_SPELL);
			return spell, texture, FloAspectBar_IsAspectActive(texture);
		end
	end
	return nil;
end

-- Returns whether the player has the aspect on or not, based on his buffs
function FloAspectBar_IsAspectActive(texture)
	local id = 0;
	for id = 0, 15 do
		if ( GetPlayerBuffTexture(id) == texture ) then
			return true;
		end
	end
	return false;
end
